import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Window // For Screen
import QtWebEngine
import QtQuick.VirtualKeyboard
import QtQuick.VirtualKeyboard.Styles
import Qt.labs.settings


import "."

Item {
    id: root
    
    /**
     * Default to 1366x768 with a pixel density of 2.91251 px/mm
     */
    readonly property int defaultWidth: 1366
    readonly property int defaultHeight: 768
    readonly property real defaultPixelDensity: 2.91251
    /**
     * Calculate scale factors based on ratio of current screen to default and density of pixels
     */
    readonly property real hScaleFactor: (Screen.width/defaultWidth)//*(Screen.pixelDensity/defaultPixelDensity)
    readonly property real vScaleFactor: (Screen.height/defaultHeight)//*(Screen.pixelDensity/defaultPixelDensity)
    readonly property real scaleFactor: Math.min(hScaleFactor, vScaleFactor)

    readonly property string googleLoginUserAgent: "https://accounts.google.com/"
    readonly property string latestUserAgent: "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36"

    property bool keyingURL: false
    property string defaultURL: webView.fixUrl(application.homeUrl)
    property int currentContentWidth: 0
    property color colorBorder: "#ba3e3e"
    property color colorBackground: "#000000"
//    property color colorUrlBar: "#ffffff"
    
    property bool fullscreen: false
    property bool tabactiveInput: false
    property bool reloadPage: false


    anchors {
        fill: parent
        leftMargin: 0
        topMargin: 0
        rightMargin: 0
        bottomMargin: 0
    }

    Connections {
        target: application

        function onCommandReceived(cmd) {
            processCommand(cmd)
        }
    }

    function processCommand(cmd) {
        var parts = cmd.split(" ")
        if (parts.length < 2) {
            console.log("invalid command", cmd)
            return
        }

        switch (parts[1]) {
        case "url":
            console.log("Go to url: " + parts[2])
            webView.url = webView.fixUrl(parts[2])
            break;
        case "goback":
            controlClicked("BrowserBack")
            break;
        case "goforward":
            controlClicked("BrowserForward")
            break;
        case "gohome":
            controlClicked("BrowserHome")
            break;
        }
    }

    Rectangle {
        color: root.colorBackground
        anchors.fill: parent
    }
    
    RowLayout {
        anchors.fill: parent
        layoutDirection: Qt.RightToLeft
        spacing: 0
        
        ColumnLayout {
            Layout.fillWidth: true
            Layout.fillHeight: true
            spacing: 1 * root.vScaleFactor

            GrowlMessage {
                id: loadFailedMessage
                message: '<img src="graphics/red_x.png" /> ' + qsTr("The Requested Page Could Not Be Loaded") //+ application.emptyString
                Layout.alignment: Qt.AlignTop
            }

            WebEngineView {
                id: webView

                property bool activeInput: false

                settings.pluginsEnabled: true

                //focus: true
                profile: netflixProfile
                onFullScreenRequested: {
                    console.log("fullscreen requested", request.toggleOn)
                    request.accept()
                }

                Layout.fillWidth: true
                Layout.fillHeight: true

                onNewWindowRequested: function(request) {
                    if (!request.userInitiated) {
                        // Avoid opening popups
                        return;
                    } else if (request.destination === WebEngineView.NewViewInTab) {
                        var tab = tabs.createEmptyTab(webView.profile);
                        tabs.visible=true;
                        console.log("opening new tab request: ", request.requestedUrl);
                        request.openIn(tab);
                    } else {
                        // Create new window just to grab the URL, then have it destroy itself
                        var newWindow = windowComponent.createObject(root);
                        request.openIn(newWindow.webEngineView);
                    }
                }
                url: root.defaultURL

                onUrlChanged: {
                    if (url.toString()) {
                        netflixProfile.updateUserAgent()
                        console.log("URL changed to " + url )
                    }
                    activeInput = false
                }

                function fixUrl(url) {
                    tabs.visible=false
                    tabactiveInput=false;
                    if(url === "") return url
                    if(url[0] === "/" || url.indexOf("file:") >= 0) {
                        // Do nothing
                        return url
                    }

                    if(url.indexOf("://") < 0) {
                        if(url.indexOf(".") < 0 || url.indexOf(" ") >= 0){
                            print("url fallback ")
                            return "https://google.com/search?q=" + url
                        }
                        else
                            return urlRewrite("http://" + url)
                    }
                    else {
                        return urlRewrite(url)
                    }
                }

                function urlRewrite(url) {
                    return url
                }

                onLoadingChanged: function(loadingInfo) {
                    if (loadingInfo.status === WebEngineView.LoadFailedStatus) {
                        console.log("Failed to load webpage", webView.url, webView.progress)
                        if(root.reloadPage===false)
                        {
                             webView.reload()
                            console.log("reload the page **********************")
                            root.reloadPage=true
                        }
                        loadFailedMessage.show(3)
                    } else if (loadingInfo.status === WebEngineView.LoadStartedStatus) {
                        // Hide keypad when going to a new page
                        //Qt.inputMethod.hide()
                        //keypadManager.hideKeypad("inputKeypad")
                    } else if (loadingInfo.status === WebEngineView.LoadSucceededStatus) {
                        webView.runJavaScript("
if (window.onbeforeunload) {
// Prevent creating dialogs as user leaves page
var original_func = window.onbeforeunload;
window.onbeforeunload = function(e) {
  original_func(e);
  return null; // Do not show a dialog
};
}
// Suppress alert messages. We didn't build the module
// required to display them (QtQuick.Dialogs) and they
// misbehave when the module is installed.
window.alert = function(msg) { console.log('Alert:', msg); }
window.prompt = function(msg) { return null; }

var TrueFitness_log_message = function(msg) {
  // Some sites (e.g. twitter) override console.log to be an empty function
  // Therefore use console.error, which is at least LESS likely to be overridden
  console.error(msg);
}

var isInputField = function(element) {
  if (!element) {
    return false;
  }

  if (element.tagName !== 'INPUT' && element.tagName !== 'TEXTAREA') {
    return false;
  }
  if (element.tagName === 'TEXTAREA') {
    return true;
  }
  if (element.tagName === 'INPUT') {
    switch (element.type) {
      case 'text':
      case 'password':
      case 'search':
      case 'email':
      case 'url':
      case 'tel':
      case 'number':
        return true;
      default:
        return false;
    }
  }

  return false;
}
/*************************espn*******************/
var isEspnInput=function(element){
    TrueFitness_log_message(element.tagName);
    if(element.tagName==='A'){
        return true;
    }
    return false;
}


var isFacebookInput = function(element) {
  // Special case for Facebook - they don't use normal input fields
  // Warning: Brittle
  if (element.tagName === 'DIV') {
    if (element.getAttribute('data-offset-key') !== null) {
      return true;
    }
    if (element.getAttribute('aria-label') === 'Search Facebook') {
      return true;
    }
    return false;
  }

  return false;
}

/* Note: We set the third parameter (useCapture) to true as this ensures that the window-level
 * events run first, rather than having to bubble up from the input element. I.e. it is
 * more efficient.
 */
window.addEventListener('focus',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isInputField(e.target)) {
    TrueFitness_log_message('@InputFocus');
  }
}, true);

window.addEventListener('blur',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isInputField(e.target)) {
    TrueFitness_log_message('@InputBlur');
  }
}, true);

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isInputField(e.target)) {
    if (e.target === document.activeElement) {
      // Only fire if this element is clicked after already gaining focus
      TrueFitness_log_message('@InputClick');
    }
  }
}, true);
/*********************plus.espn.com****************/
if (window.location.href.indexOf('plus.espn.com') >= 0) {

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;

  }
if(isEspnInput(e.target)){
TrueFitness_log_message('@InputFocus');
}else {
    if (!isInputField(e.target)) {
      TrueFitness_log_message('@InputBlur');
    }
  }

}, true);

}

/******************espn*********************/

if (window.location.href.indexOf('espn.com') >= 0) {

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;

  }
if(isEspnInput(e.target)){
TrueFitness_log_message('@InputFocus');
}else {
    if (!isInputField(e.target)) {
      TrueFitness_log_message('@InputBlur');
    }
  }

}, true);

}


// -- Special cases for Facebook --
// Facebook does not use normal input fields, but DIV-based react.js fields
if (window.location.href.indexOf('facebook.com') >= 0) {

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isFacebookInput(e.target)) {
    TrueFitness_log_message('@InputFocus');
  } else {
    // Anything that is not a FB input field, or traditional input
    // field, is treated as a blur event.
    if (!isInputField(e.target)) {
      TrueFitness_log_message('@InputBlur');
    }
  }
}, true);

}
// -- End Facebook special case --

// Check if an input field is already selected, and show keypad if so
if (false) { // Disable for now
  if (document && document.activeElement) {
    if (isInputField(document.activeElement)) {
      TrueFitness_log_message('@InputFocus');
    }
  }
}
/// NOTE: Used to work around the old version of chromium in Qt 6.2.4 not having the .at function
// Found at https://stackoverflow.com/a/70557417
function at(n) {
    // ToInteger() abstract op
    n = Math.trunc(n) || 0;
    // Allow negative indexing from the end
    if (n < 0) n += this.length;
    // OOB access is guaranteed to return undefined
    if (n < 0 || n >= this.length) return undefined;
    // Otherwise, this is just normal property access
    return this[n];
}

const TypedArray = Reflect.getPrototypeOf(Int8Array);
for (const C of [Array, String, TypedArray]) {
    Object.defineProperty(C.prototype, 'at',
                          { value: at,
                            writable: true,
                            enumerable: false,
                            configurable: true });
}
");
                    }
                }

                onJavaScriptConsoleMessage: function(level, message, lineNumber, sourceID) {
                    console.log("onJavaScriptConsoleMessage(level, message, lineNumber, sourceID): " + level + ", " + message + ", " + lineNumber + ", " + sourceID);

                    if (message[0] === "@") {
                        if (message === "@InputFocus" || message === "@InputClick") {
                            activeInput = true
                        } else if (message === "@InputBlur") {
                            activeInput = false
                        }
                    }
                }

                transformOrigin: Item.TopLeft
                smooth: true
            } // end WebEngineView
        } // end ColumnLayout
    } // end RowLayout

    StackView {
        id: tabs
        visible: false
        function createEmptyTab(profile) {
            if(depth > 0)
            {
                console.log("Clearing old tabs");
                clear();
                tabactiveInput = false;
            }
            tabactiveInput = true;

            push(tabComponent);
            var newWindow = get(0);
            return newWindow;
        }

        function getActiveView() {
            if(tabs.depth > 0) {
                return get(0);
            } else {
                return undefined;
            }
        }

        anchors.fill: parent


        Component {
            id: tabComponent

            WebEngineView {
                id: webEngineView

                focus: true
                profile: netflixProfile
                settings.autoLoadImages: true
                settings.javascriptEnabled: true
                settings.errorPageEnabled: true
                settings.pluginsEnabled: true
                settings.fullScreenSupportEnabled: true
                settings.autoLoadIconsForPage: true
                settings.touchIconsEnabled: true
                settings.webRTCPublicInterfacesOnly: true

                onWindowCloseRequested: {
                    console.log("close the window", tabs.currentIndex);

                    tabs.removeTab(tabs.currentIndex);
                    tabs.visible=false;
                    tabactiveInput=false;
                }

                onUrlChanged: {
                    if (url.toString()) {
                        netflixProfile.updateUserAgent()
                        console.log("URL changed to " + url)
                    }
                }
            }
        }
    }
   

    property Component windowComponent: Item {
        visible: false

        property WebEngineView webEngineView: webEngineView_
        WebEngineView {
            id: webEngineView_
            profile: netflixProfile
            anchors.fill: parent
            onUrlChanged: {
                if (!url.toString()) {
                    return
                }
                console.log("Opening", url, "in current window rather than new window")
                webView.url = url
                netflixProfile.updateUserAgent()
                parent.destroy()
            }
        }
    }

    WebEngineProfile {
        id: privateBrowsing
        offTheRecord: false
        //persistentCookiesPolicy: WebEngineProfile.NoPersistentCookies
    }

    WebEngineProfile {
        id: netflixProfile
        // Netflix requires at least offTheRecord to be false for its encryption to work
        // The other properties are an attempt to tighten the number of other items that get stored
        offTheRecord: false
        persistentCookiesPolicy: WebEngineProfile.NoPersistentCookies
        persistentStoragePath: application.persistentStoragePath
        httpCacheType: WebEngineProfile.MemoryHttpCache
        cachePath: ""

        function updateUserAgent() {
            var currentWebView;
            if(tabs.visible) {
                currentWebView = tabs.getActiveView();
            } else {
                currentWebView = webView
            }

            if(currentWebView !== undefined) {
                var newHttpUserAgent = (currentWebView.url.toString().indexOf("accounts.google.com") >= 0) ? googleLoginUserAgent : latestUserAgent;
                if(newHttpUserAgent !== httpUserAgent) {
                    httpUserAgent = (currentWebView.url.toString().indexOf("accounts.google.com") >= 0) ? googleLoginUserAgent : latestUserAgent;
                    console.log("Setting user agent to ", httpUserAgent);
                }
            } else {
                console.log("currentWebView is undefined: ", currentWebView);
            }
        }

        httpUserAgent: updateUserAgent()
    }

    ListModel {
        id: controlsList
        ListElement {
            name: "BrowserBack"
            label: ""
            iconPath: "graphics/ui3_0/back_triangle_white.png"
        }
        ListElement {
            name: "BrowserForward"
            label: ""
            iconPath: "graphics/ui3_0/forwrd_triangle_white.png"
        }
        ListElement {
            name: "BrowserHome"
            label: ""
            iconPath: "graphics/ui3_0/Home_icon.png"
        }
        ListElement {
            name: "BrowserFullScreen"
            label: ""
            iconPath: "graphics/ui3_0/fullscreen_icon.png"
        }
    }

    function controlClicked(name)
    {
        if(name === "BrowserBack") {
            if(webView.canGoBack) {
                webView.goBack()
            }
        } else if(name === "BrowserForward") {
            if(webView.canGoForward) {
                webView.goForward()
            }
        } else if(name === "BrowserHome") {
            webView.url = root.defaultURL
        } else if(name === "BrowserFullScreen") {
            root.fullscreen = !root.fullscreen;
        }
    }

    TrueKeyboard {
        id: keyboard
        width: root.width * 0.75
        anchors.horizontalCenter: root.horizontalCenter
        active: Qt.inputMethod.visible | tabactiveInput
        z: 999999
    }

    Connections {
        target: InputContext.inputEngine
        function onPreviousKeyChanged() {
            if(webView.activeInput) {
                if(InputContext.shiftActive) {
                    InputContext.shiftActive = false
                }
            }
        }
    }

}
